//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	DerFil
// Auteur			:	PERRICHON Pierre
// Date				:	29/12/2018
// Version			:	1.0
//
// Sujet			: 	Dérivée filtrée Td.p/1+Td/Kd.p
//						
//
// Paramétrage		:	Constante de temps Td du dérivateur (s)
//						Gain Kd du dérivateur
//						Valeur initiale de l'entrée de DerFil à t=0
//
//---------------------------------------------------------------------------------------
// ATTENTION : NE PAS MODIFIER LES APPELLATIONS Kdxx ET Tdxx dans ce source (bug Scilab)
//---------------------------------------------------------------------------------------
// Autres données	:	Le code objet est celui du composant CLR dans la Scilab
//						
//						
// Sorties			:	Dérivée filtrée
//						
//						
//						
// Interface IHM	:	DerFil.sci
// Code objet		;	SCI/modules/scicos_blocks/src/c/csslti4.c (Type 4) 
//
// Programme de test:	DerFil.zos
//
// Fonction			:	DerFil
// Auteur			:	PERRICHON Pierre
// Date				:	19/01/2024
// Version			:	1.1
//					: 	On remplace la fonction de transfert H2=Td*s/(1+Td/Kd*s)
//					:	par H2=Kd*Td*s/(Kd+Td*s) pour pouvoir annuler la dérivée avec Kd=0.
//						Représentation inchangée dans le bloc DerFil
//------------------------------------------------------------------------------------------------------------------------
function [sciblk, s1, s2] = DerFil(job,sciblk,e1)

    s1=[]; s2=[];
	s=%s;
	
	Module="DerFil";
	[linenum, fun] = where()
	editSegPalBlock=~or(fun == "do_eval")	// Détertime si mode simulation ou édition du composant par double clic
	
    
    select job
	
		case "set" then
		
			graphics=sciblk.graphics;
			exprs=graphics.exprs
			model=sciblk.model;
			
			// Analyse préliminaire des expressions
			//-------------------------------------
			[bdiag,DefExp]=CheckExprs(exprs) 		
			if ~bdiag then 
						// Certaines variables ne sont pas définies
						//-----------------------------------------
						if SegPalError(2, Module, DefExp)==2 then return; end
			end
					
			x0=model.state
			rpar=model.rpar
		
			Tdxx=rpar(5);
			Kdxx=rpar(6);
			V0=rpar(7);
			
			ns=prod(size(x0));
			nin=1;nout=1;
			%scicos_context=%scicos_context; //copy the semi-global variable locally
			%scicos_context.s=%s //add s definition to the context

			Titre="Paramètres du filtre dérivateur DerFil"
			
			labels=["Constante de temps du dérivateur Td (s)";...
					"Gain du dérivateur Kd";...
					"Entrée initiale du filtre à t=0"];
					

			if getlanguage()=="en_US" then		
			
				Titre="DerFil  - Filtered derivative parameters"
					
				labels=["Derivative time constant Td (s)";...
					"Derivative gain Kd";...
					"Deivative initial value at t=0"];
					
			end
								
			while %t do
							
				[ok,Tdxx, Kdxx, V0,exprs]=scicos_getvalue(Titre,labels,...
									list("vec",1,"vec",1,"vec",1),exprs)
				
				if ~ok then return; end
				
				//Analyse du paramétrage
				//----------------------			
				[bdiag,DefExp]=CheckExprs(exprs)
				if ~bdiag then SegPalError(2, Module, DefExp)
					else
							DefExp=[]
							
							//...Pas de test sur les variables

							if DefExp==[] then break; end
									
							SegPalError(1, Module, DefExp);
				end
				
			end	
								
			// Mise en place du filtre dérivateur
			//----------------------------------
				
			graphics.exprs=exprs;			

			//H2=Tdxx*s/(1+Tdxx/Kdxx*s)				// Version 1.0	
			A=0;B=0;C=0;D=0;x0=-D*V0;

			if Kdxx<>0 then
				H2=Kdxx*Tdxx*s/(Kdxx+Tdxx*s)			// Version 1.1
				Sys=tf2ss(H2)	
				A=Sys.A
				B=Sys.B
				C=Sys.C
				D=Sys.D
				x0=-D*V0/C
			end
		
			rpar=[A(:);B(:);C(:);D(:);Tdxx;Kdxx;V0]
					
			model.state=x0
			model.rpar=rpar
			sciblk.graphics=graphics;
			sciblk.model=model
					
			//Affichage de la constante de temps du filtre dans la boîte fonctionnelle
			//------------------------------------------------------------------------
			TdDisplay=string(exprs(1))
			KdDisplay=string(exprs(2))

			style_properties = ["blockWithLabel";
									"verticalLabelPosition=middle";
									"verticalAlign=left";
									"fillColor=#abcdee";
									"fontColor=blue";
									"fontSize=15";															
									"displayedLabel=$\frac{" + TdDisplay + ".p} {1 +" + TdDisplay + "/" + KdDisplay + ".p }$"];
			sciblk.graphics.style = strcat(style_properties,";");			
		
		case "define" then
			Tdxx=1;
			Kdxx=10;
			V0=0.;
			
			//H2=Tdxx*s/(1+Tdxx/Kd*s)				// Version 1.0	
			H2=Kdxx*Tdxx*s/(Kdxx+Tdxx*s)				// Version 1.1
			Sys=tf2ss(H2)	
			
			A=Sys.A
			B=Sys.B
			C=Sys.C
			D=Sys.D
			x0=-D*V0/C

			model=scicos_model()
			
			model.sim=list("csslti4",4)		// Fonction Scilab en c dans modules\scicos_blocks
			
			model.in=1
			model.out=1
			model.state=x0
			model.rpar=[A(:);B(:);C(:);D(:);Tdxx;Kdxx;V0]
			model.blocktype="c"
			model.dep_ut=[%f %t]

			exprs=string([Tdxx;Kdxx;V0])
			
			sciblk = standard_define([5 3],model,exprs);
			
			/*style_properties2 = ["blockWithLabel";
                       "verticalLabelPosition=middle";
                       "verticalAlign=left";
						"fillColor=#abcdee";
						"fontColor=blue";
						"fontSize=15";						
						"displayedLabel=$\frac{Td.p}{1+ Td/Kd.p }$"];
			sciblk.graphics.style = strcat(style_properties,";");*/
			
			TdDisplay=string(exprs(1))
			KdDisplay=string(exprs(2))

			style_properties = ["blockWithLabel";
									"verticalLabelPosition=middle";
									"verticalAlign=left";
									"fillColor=#abcdee";
									"fontColor=blue";
									"fontSize=15";															
									"displayedLabel=$\frac{" + TdDisplay + ".p} {1 +" + TdDisplay + "/" + KdDisplay + ".p }$"];
			sciblk.graphics.style = strcat(style_properties,";");			
				
    end
	
endfunction
