//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	IntLim
// Auteur			:	PERRICHON Pierre
// Date				:	01/05/2014
// Version			:	1.0
//
// Sujet			: 	Intégrateurs avec réinitialisation et saturations externes
//
// Paramétrage		:	Constante de temps Ti de l'intégrateur (s)
//						Valeur initiale de l'intégrateur à t=0
//						Avec ou sans réinitialisation programmable (0 ou 1)
//						Avec ou sans saturations externes (0 ou 1)
//
// Autres données	:	
//						
//						
// Sorties			:	Intégrateurs avec saturation
//						
//						
// Modification		:
// Auteur			:	PERRICHON Pierre
// Date				:	25/08/2021
// Version			:	1.1
//
// Description		!	Réarragement des E/S de la boite fonctionnelles
//						Entrée "in" vectorisée n lignes x 1 colonne
//						Réinitialisation : 1 scalaire 1x1 pour tous les intégrateurs
//						Limitations UL et LL 1 scalire 1x1 pour tous les intégrateurs
//						Sortie out vectorisée n lignes x 1 colonne						
//						
// Interface IHM	:	IntLim.sci
// Code objet		;	IntLimC.c 
//
// Programmes de test:	IntLim.zos Les_Trois_SYSCOP.zcos
//
//------------------------------------------------------------------------------------------------------------------------

#include "Common.h"
#include <sciprint.h>


void  IntLimC(scicos_block *Cblk, scicos_flag flag)
{
	static char *Module="IntLimC";
	
	char MesErr[120];
	char *Label;
	int NumBlockErr;
	
	int i = 0;
	int n = GetInPortRows(Cblk,1); 
	
	int reinit=Cblk->ipar[0];
	int sature=Cblk->ipar[1];
	double UL,LL;			// Upper Limiter - Lower Limiter - Limiteurs de l'intégrateur

	int numport;
	
	double *in,*in2,*out,*x,*xd,Tau;
 	
	if (sature) {
					numport=2;
					if (reinit) numport=3;
					 
					UL=*GetRealInPortPtrs(Cblk,numport);
					LL=*GetRealInPortPtrs(Cblk,numport+1);
					
					if (LL>UL) {
						//set_block_error(-1); 	// Données d'entrée inappropriées : voir help c_utils
						NumBlockErr=get_block_number();
						Label=Cblk->label;
						sprintf(MesErr,"Module %s - Block %i \nLower limiter LL=%f greater than the Upper UL=%f",Module,NumBlockErr,LL,UL);
						Coserror(MesErr);
					}
	}
  
	switch(flag) {
	
		case DerivativeState: 
			// Evaluation de la fonction C 
			//----------------------------
			in = GetRealInPortPtrs(Cblk,1);  
			xd = GetDerState(Cblk);
			
			Tau=Cblk->rpar[0];
			if (Tau==0.) Tau=1.e-10;
			
			for(i=0; i<n; ++i) xd[i] = in[i]/Tau;
			break;

		case ReInitialization:
		case OutputUpdate:	
			// Evaluation de la fonction F
			//----------------------------
			x = GetState(Cblk);
			out = GetRealOutPortPtrs(Cblk,1);

			for(i=0; i<n; ++i) {
							x[i] = Limiteur(x[i],sature,UL,LL); 	// Limitation de la matrice d'état
							if(fabs(x[i])<1.e-15) x[i]=0.;			// Saturation - Rajout PPe le 18.06.2019
							out[i]=x[i];
			}
			break;
			
		case StateUpdate:
			// Evaluation de la fonction Dx
			//-----------------------------
			x = GetState(Cblk);
			in2 = GetRealInPortPtrs(Cblk,2); 
			for(i=0; i<n; ++i) x[i] = Limiteur(*in2,sature,UL,LL);	// Modif PPe le 24.08.2021 in2 devient un scalaire 1x1
			break;
	} 
}